"""
Type annotations for boto3.docs.waiter module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.hooks import BaseEventHooks
from botocore.model import ServiceModel
from botocore.waiter import WaiterModel

from .base import NestedDocumenter

class WaiterResourceDocumenter(NestedDocumenter):
    def __init__(
        self,
        resource: Any,
        service_waiter_model: WaiterModel,
        root_docs_path: str,
    ) -> None: ...
    def document_resource_waiters(self, section: DocumentStructure) -> None: ...

def document_resource_waiter(
    section: DocumentStructure,
    resource_name: str,
    event_emitter: BaseEventHooks,
    service_model: ServiceModel,
    resource_waiter_model: Any,
    service_waiter_model: WaiterModel,
    include_signature: bool = ...,
) -> None: ...



"""
Type annotations for boto3.dynamodb.conditions module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any, Literal, NamedTuple, Pattern, TypedDict

ATTR_NAME_REGEX: Pattern[str]

class _ConditionBaseExpressionType(TypedDict):
    format: str
    operator: str
    values: list[Any]

class ConditionBase:
    expression_format: str
    expression_operator: str
    has_grouped_values: bool
    def __init__(self, *values: Any) -> None: ...
    def __and__(self, other: ConditionBase) -> And: ...
    def __or__(self, other: ConditionBase) -> Or: ...
    def __invert__(self) -> Not: ...
    def get_expression(self) -> _ConditionBaseExpressionType: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

class AttributeBase:
    def __init__(self, name: str) -> None:
        self.name: str = ...

    def __and__(self, value: Any) -> Any: ...
    def __or__(self, value: Any) -> Any: ...
    def __invert__(self) -> Any: ...
    def eq(self, value: Any) -> Equals: ...
    def lt(self, value: Any) -> LessThan: ...
    def lte(self, value: Any) -> LessThanEquals: ...
    def gt(self, value: Any) -> GreaterThan: ...
    def gte(self, value: Any) -> GreaterThanEquals: ...
    def begins_with(self, value: Any) -> BeginsWith: ...
    def between(self, low_value: Any, high_value: Any) -> Between: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

class ConditionAttributeBase(ConditionBase, AttributeBase):
    def __init__(self, *values: Any) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

class ComparisonCondition(ConditionBase):
    expression_format: Literal["{0} {operator} {1}"]  # type: ignore [override]

class Equals(ComparisonCondition):
    expression_operator: Literal["="]  # type: ignore [override]

class NotEquals(ComparisonCondition):
    expression_operator: Literal["<>"]  # type: ignore [override]

class LessThan(ComparisonCondition):
    expression_operator: Literal["<"]  # type: ignore [override]

class LessThanEquals(ComparisonCondition):
    expression_operator: Literal["<="]  # type: ignore [override]

class GreaterThan(ComparisonCondition):
    expression_operator: Literal[">"]  # type: ignore [override]

class GreaterThanEquals(ComparisonCondition):
    expression_operator: Literal[">="]  # type: ignore [override]

class In(ComparisonCondition):
    expression_operator: Literal["IN"]  # type: ignore [override]
    has_grouped_values: bool

class Between(ConditionBase):
    expression_operator: Literal["BETWEEN"]  # type: ignore [override]
    expression_format: Literal["{0} {operator} {1} AND {2}"]  # type: ignore [override]

class BeginsWith(ConditionBase):
    expression_operator: Literal["begins_with"]  # type: ignore [override]
    expression_format: Literal["{operator}({0}, {1})"]  # type: ignore [override]

class Contains(ConditionBase):
    expression_operator: Literal["contains"]  # type: ignore [override]
    expression_format: Literal["{operator}({0}, {1})"]  # type: ignore [override]

class Size(ConditionAttributeBase):
    expression_operator: Literal["size"]  # type: ignore [override]
    expression_format: Literal["{operator}({0})"]  # type: ignore [override]

class AttributeType(ConditionBase):
    expression_operator: Literal["attribute_type"]  # type: ignore [override]
    expression_format: Literal["{operator}({0}, {1})"]  # type: ignore [override]

class AttributeExists(ConditionBase):
    expression_operator: Literal["attribute_exists"]  # type: ignore [override]
    expression_format: Literal["{operator}({0})"]  # type: ignore [override]

class AttributeNotExists(ConditionBase):
    expression_operator: Literal["attribute_not_exists"]  # type: ignore [override]
    expression_format: Literal["{operator}({0})"]  # type: ignore [override]

class And(ConditionBase):
    expression_operator: Literal["AND"]  # type: ignore [override]
    expression_format: Literal["({0} {operator} {1})"]  # type: ignore [override]

class Or(ConditionBase):
    expression_operator: Literal["OR"]  # type: ignore [override]
    expression_format: Literal["({0} {operator} {1})"]  # type: ignore [override]

class Not(ConditionBase):
    expression_operator: Literal["NOT"]  # type: ignore [override]
    expression_format: Literal["({operator} {0})"]  # type: ignore [override]

class Key(AttributeBase): ...

class Attr(AttributeBase):
    def ne(self, value: Any) -> NotEquals: ...
    def is_in(self, value: Any) -> In: ...
    def exists(self) -> AttributeExists: ...
    def not_exists(self) -> AttributeNotExists: ...
    def contains(self, value: Any) -> Contains: ...
    def size(self) -> Size: ...
    def attribute_type(self, value: Any) -> AttributeType: ...

class BuiltConditionExpression(NamedTuple):
    condition_expression: str
    attribute_name_placeholders: dict[str, str]
    attribute_value_placeholders: dict[str, Any]

class ConditionExpressionBuilder:
    def __init__(self) -> None: ...
    def reset(self) -> None: ...
    def build_expression(
        self, condition: ConditionBase, is_key_condition: bool = ...
    ) -> BuiltConditionExpression: ...



"""
Type annotations for boto3.resources.params module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any, Pattern

from boto3.resources.base import ServiceResource
from boto3.resources.model import Request

INDEX_RE: Pattern[str]

def get_data_member(parent: ServiceResource, path: str) -> dict[str, Any] | None: ...
def create_request_parameters(
    parent: ServiceResource,
    request_model: Request,
    params: dict[str, Any] | None = ...,
    index: int | None = ...,
) -> dict[str, Any]: ...
def build_param_structure(
    params: dict[str, Any], target: str, value: Any, index: int | None = ...
) -> None: ...



"""
Type annotations for boto3.docs.service module.

Copyright 2024 Vlad Emelianov
"""

from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.docs.service import ServiceDocumenter as BaseServiceDocumenter
from botocore.session import Session

class ServiceDocumenter(BaseServiceDocumenter):
    EXAMPLE_PATH: str
    sections: list[str]
    def __init__(self, service_name: str, session: Session, root_docs_path: str) -> None: ...
    def document_service(self) -> bytes: ...
    def client_api(self, section: DocumentStructure) -> None: ...
    def resource_section(self, section: DocumentStructure) -> None: ...



"""
Type annotations for boto3.docs.action module.

Copyright 2024 Vlad Emelianov
"""

from boto3.resources.model import Action
from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.hooks import BaseEventHooks
from botocore.model import ServiceModel

from .base import NestedDocumenter

PUT_DATA_WARNING_MESSAGE: str
WARNING_MESSAGES: dict[str, dict[str, str]]
IGNORE_PARAMS: dict[str, dict[str, list[str]]]

class ActionDocumenter(NestedDocumenter):
    def document_actions(self, section: DocumentStructure) -> None: ...

def document_action(
    section: DocumentStructure,
    resource_name: str,
    event_emitter: BaseEventHooks,
    action_model: Action,
    service_model: ServiceModel,
    include_signature: bool = ...,
) -> None: ...
def document_load_reload_action(
    section: DocumentStructure,
    action_name: str,
    resource_name: str,
    event_emitter: BaseEventHooks,
    load_model: Action,
    service_model: ServiceModel,
    include_signature: bool = ...,
) -> None: ...



"""
Type annotations for boto3.ec2.deletetags module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

from botocore.hooks import BaseEventHooks

def inject_delete_tags(event_emitter: BaseEventHooks, **kwargs: Any) -> None: ...
def delete_tags(self: Any, **kwargs: Any) -> None: ...



"""
Type annotations for boto3.resources.collection module.

Copyright 2024 Vlad Emelianov
"""

import logging
from typing import Any, Iterator, TypeVar

from boto3.resources.base import ServiceResource
from boto3.resources.factory import ResourceFactory
from boto3.resources.model import Collection
from boto3.resources.response import ResourceHandler
from boto3.utils import ServiceContext
from botocore.hooks import BaseEventHooks

logger: logging.Logger

_ResourceCollection = TypeVar("_ResourceCollection", bound=ResourceCollection)

class ResourceCollection:
    def __init__(
        self, model: Collection, parent: ServiceResource, handler: ResourceHandler, **kwargs: Any
    ) -> None: ...
    def __iter__(self) -> Iterator[Any]: ...
    def pages(self) -> Iterator[list[Any]]: ...
    def all(self: _ResourceCollection) -> _ResourceCollection: ...
    def filter(self: _ResourceCollection, **kwargs: Any) -> _ResourceCollection: ...
    def limit(self: _ResourceCollection, count: int) -> _ResourceCollection: ...
    def page_size(self: _ResourceCollection, count: int) -> _ResourceCollection: ...

class CollectionManager:
    def __init__(
        self,
        collection_model: Collection,
        parent: ServiceResource,
        factory: ResourceFactory,
        service_context: ServiceContext,
    ) -> None: ...
    def iterator(self, **kwargs: Any) -> ResourceCollection: ...
    def all(self) -> ResourceCollection: ...
    def filter(self, **kwargs: Any) -> ResourceCollection: ...
    def limit(self, count: int) -> ResourceCollection: ...
    def page_size(self, count: InterruptedError) -> ResourceCollection: ...
    def pages(self) -> list[ServiceResource]: ...

class CollectionFactory:
    def load_from_definition(
        self,
        resource_name: str,
        collection_model: Collection,
        service_context: ServiceContext,
        event_emitter: BaseEventHooks,
    ) -> CollectionManager: ...



"""
Type annotations for boto3.s3.transfer module.

Copyright 2024 Vlad Emelianov
"""

import logging
from types import TracebackType
from typing import Any, Callable, Mapping, TypeVar

from botocore.client import BaseClient
from botocore.config import Config
from s3transfer.manager import TransferConfig as S3TransferConfig
from s3transfer.manager import TransferManager
from s3transfer.subscribers import BaseSubscriber
from s3transfer.utils import OSUtils

_R = TypeVar("_R")

KB: int
MB: int

logger: logging.Logger = ...

def create_transfer_manager(
    client: BaseClient,
    config: TransferConfig,
    osutil: OSUtils | None = ...,
) -> TransferManager: ...
def has_minimum_crt_version(minimum_version: str) -> bool: ...

class TransferConfig(S3TransferConfig):
    ALIAS: dict[str, str]

    def __init__(
        self,
        multipart_threshold: int = ...,
        max_concurrency: int = ...,
        multipart_chunksize: int = ...,
        num_download_attempts: int = ...,
        max_io_queue: int = ...,
        io_chunksize: int = ...,
        use_threads: bool = ...,
        max_bandwidth: int | None = ...,
        preferred_transfer_client: str = ...,
    ) -> None:
        self.use_threads: bool

    def __setattr__(self, name: str, value: int) -> None: ...

class S3Transfer:
    ALLOWED_DOWNLOAD_ARGS: list[str]
    ALLOWED_UPLOAD_ARGS: list[str]
    def __init__(
        self,
        client: BaseClient | None = ...,
        config: Config | None = ...,
        osutil: OSUtils | None = ...,
        manager: TransferManager | None = ...,
    ) -> None: ...
    def upload_file(
        self,
        filename: str,
        bucket: str,
        key: str,
        callback: Callable[[int], Any] | None = ...,
        extra_args: Mapping[str, Any] | None = ...,
    ) -> None: ...
    def download_file(
        self,
        bucket: str,
        key: str,
        filename: str,
        extra_args: Mapping[str, Any] | None = ...,
        callback: Callable[[int], Any] | None = ...,
    ) -> None: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        tb: TracebackType | None,
    ) -> None: ...

class ProgressCallbackInvoker(BaseSubscriber):
    def __init__(self, callback: Callable[[int], Any]) -> None: ...
    # FIXME: signature incompatible with BaseSubscriber
    def on_progress(  # type: ignore [override]
        self, bytes_transferred: int, **kwargs: Any
    ) -> None: ...



"""
Type annotations for boto3.docs.resource module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.session import Session

from .base import BaseDocumenter

class ResourceDocumenter(BaseDocumenter):
    def __init__(self, resource: Any, botocore_session: Session, root_docs_path: str) -> None: ...
    def document_resource(self, section: DocumentStructure) -> None: ...

class ServiceResourceDocumenter(ResourceDocumenter):
    @property
    def class_name(self) -> str: ...
